%{
#include <blocxx/Exception.hpp>
#include <limal/Exception.hpp>
#include <blocxx/IOException.hpp>
#include <blocxx/StringStream.hpp>
#include <blocxx/Array.hpp>
#include <stdexcept>
#include <typeinfo>
%}


#ifdef SWIGPERL5
%{

static SWIGINLINE void
LIMAL_buildExceptionHash(SWIG_MAYBE_PERL_OBJECT HV * hash, const blocxx::Exception &e) {

    SV * value = newSVpv(e.type(), 0);
    hv_store(hash, "type", 4, value, 0);

    value = newSViv(e.getErrorCode());
    hv_store(hash, "code", 4, value, 0);

    value = newSVpv(e.getMessage(), 0);
    hv_store(hash, "message", 7, value, 0);

    value = newSVpv(e.getFile(), 0);
    hv_store(hash, "file", 4, value, 0);

    value = newSViv(e.getLine());
    hv_store(hash, "line", 4, value, 0);

    if(e.getSubException()) {

        HV *subHash = newHV();

#ifdef PERL_OBJECT
        LIMAL_buildExceptionHash(pPerl, subHash, *(e.getSubException()));
#else
        LIMAL_buildExceptionHash(subHash, *(e.getSubException()));
#endif

        hv_store(hash, "subexception", 12, newRV_noinc((SV*)subHash), 0);
    } 
}

static SWIGINLINE void
LIMAL_croak(SWIG_MAYBE_PERL_OBJECT const blocxx::Exception &e) {

    HV * hash  = newHV();

#ifdef PERL_OBJECT
    LIMAL_buildExceptionHash(pPerl, hash, e);
#else
    LIMAL_buildExceptionHash(hash, e);
#endif

    sv_setsv(perl_get_sv("@", TRUE), newRV_noinc((SV*)hash));
}

#define LIMAL_exception(a)  { LIMAL_croak(a); SWIG_fail; }


static SWIGINLINE void
LIMAL_croak2(SWIG_MAYBE_PERL_OBJECT const std::exception &e) {

    HV * hash  = newHV();

    SV * value = newSVpv(typeid(e).name(), 0);
    hv_store(hash, "type", 4, value, 0);

    value = newSVpv(e.what(), 0);
    hv_store(hash, "message", 7, value, 0);

    sv_setsv(perl_get_sv("@", TRUE), newRV_noinc((SV*)hash));
}

#define LIMAL_exception2(a)  { LIMAL_croak2(a); SWIG_fail; }

%}

#else
%include <exception.i>
#endif


%exception {
        try {
                $action
        } catch(const blocxx::Exception &e) {
            LIMAL_exception(e);
        } catch(const std::exception &e) {
            LIMAL_exception2(e);
        } catch(...) {
            SWIG_exception(SWIG_UnknownError, "UnknownError");
            SWIG_fail;
        }
}
