/* LiMaL callback wrapper module */
%module limal_Callback

/*
 * include the destination language specific
 * wrapper code, e.g. perl5/_limal_Callback.i
 */
%include _limal_Callback.i

/*
** ******************************************************
** ********** Usage Example: ****************************
** ******************************************************
**
** Write your Foo.i file and include the header
** file(s) containing your interfaces:
**

%module Foo

**
** - include the limal callback wrapper module:
**

%include limal
%include limal_Callback.i

**
** - and header file containing the interface to wrap:
**

%include Foo.hpp

**
** let's assume it contains for this declarations:
**
**     #include <limal/CallbackBase.hpp>
**     namespace Foo {
**
**        class FooCBRequest { ... };
**        class FooCBResult1 { ... };
**        class FooCBResult2 { ... };
**
**        typedef FooCB1 limal::CallbackBase(FooCBRequest,FooCBResult1);
**        typedef FooCB2 limal::CallbackBase(FooCBRequest,FooCBResult2);
**
**        int my_foobar_function_using_cb1(FooCB1 *cb);
**        int my_foobar_function_using_cb2(FooCB2 *cb);
**     }
**

**
** To generate the callback wrappers, use the following macros:
**
** - generate swig type conversions for each request and result type
**

specialize_limal_callback_request(Foo::FooCBRequest)
specialize_limal_callback_result(Foo::FooCBResult1)
specialize_limal_callback_result(Foo::FooCBResult2)

**
** - specialize and instantiate each callback (calls %template):
**

specialize_limal_callback_template(FooCB1,Foo::FooCBRequest,Foo::FooCBResult1);
specialize_limal_callback_template(FooCB2,Foo::FooCBRequest,Foo::FooCBResult2);

**
**  !! NOTE the explicit specification of the "Foo" namespace for
**     FooCBRequest/FooCBResult's but _NOT_ for the callback names
**     FooCB1/FooCB2, that are used in %template instantiation only !!
**

**
** Alternatively, if you have only one callback, use the shortcut
** macro specialize_limal_callback, that calls the above macros:
**
specialize_limal_callback(FooCB1, Foo::FooCBRequest, Foo::FooCBResult);

*/

